Attribute VB_Name = "Swvb32_main"
'//--------------------------------------------------------------------------
'// Copyright (C) 1992,1993,1994,1995,1996,1997,1998 Dallas Semiconductor Corporation.
'// All rights Reserved. Printed in U.S.A.
'// This software is protected by copyright laws of
'// the United States and of foreign countries.
'// This material may also be protected by patent laws of the United States
'// and of foreign countries.
'// This software is furnished under a license agreement and/or a
'// nondisclosure agreement and may only be used or copied in accordance
'// with the terms of those agreements.
'// The mere transfer of this software does not imply any licenses
'// of trade secrets, proprietary technology, copyrights, patents,
'// trademarks, maskwork rights, or any other form of intellectual
'// property whatsoever. Dallas Semiconductor retains all ownership rights.
'//--------------------------------------------------------------------------
'

Option Explicit

'// Global Variable Declaration

Global state_buffer(15360) As Byte
Global PortNum As Integer
Global PortType As Integer
Global SHandle As Long
Global crc16 As Long
Global ROM(8) As Integer
Public Const Channel_Access As Integer = &HF5
Public Const write_status As Integer = &H55
Public Const family As Integer = &H37 '// family code for DS1977
Declare Function GetTickCount Lib "kernel32" () As Long
Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)



' Utility function for DoCrc16
Function odd_parity(ByVal dat As Long) As Long
    Dim n As Long
    Dim I As Long

    n = 0
    For I = 0 To 7
        n = n + dat And 1
        dat = (dat And 65534) / 2
    Next I
    odd_parity = n And 1
End Function

' Compute CRC16 for 1-Wire comunication integrety checking
Function DoCrc16(ByVal dat As Long) As Long
    dat = (dat Xor (crc16 And 255)) And 255
    crc16 = (crc16 And 65280) / 256
    If (odd_parity(dat) = 1) Then crc16 = crc16 Xor 49153
    dat = (dat * 64) And 65535
    crc16 = crc16 Xor dat
    dat = (dat * 2) And 65535
    crc16 = crc16 Xor dat
    crc16 = crc16
    DoCrc16 = crc16
End Function
' ------------------------------------------------------------------------------
' Read a page from DS1977 with CRC verification
'
'   SHandle - session handle
'   ROMID() - Array of Integers representing DS1977 ROM ID to read
'   psw() - Array of 8 Bytes representing the password
'   page - page number to read 0 - 510
'   rd_cont - read continue flag, TRUE if last operation was to read previous page
'             provides a speed improviment when reading contigious pages
'   data() - Array of Bytes to hold 64 byte page data
'
'  Returns True if read was successful
'
Function ReadPageCRC77(SHandle As Long, ByRef ROMID() As Integer, ByRef psw() As Byte, page As Integer, rd_cont As Boolean, ByRef data() As Byte) As Boolean
    Dim buf(80) As Byte
    Dim Dummy As Long
    Dim addr As Integer
    Dim send_len As Integer
    Dim I As Integer
    
    ' set device ROM ID
    Dummy = TMRom(SHandle, state_buffer(0), ROMID(0))
    
    ' if not read continue then access the device
    If Not rd_cont Then
        ' select the device
        If (TMAccess(SHandle, state_buffer(0)) <> 1) Then
            ReadPageCRC77 = False
            Exit Function
        End If
            
        ' read block to set to start read-page
        buf(0) = &H69  ' read memory with password command
        addr = page * 64
        buf(1) = addr And &HFF  ' TA1
        buf(2) = Int(addr / 256) And &HFF ' TA2
        send_len = 3
    
        ' calculate CRC16 on first chunk of packet
        crc16 = 0
        For I = 0 To send_len - 1
            DoCrc16 (buf(I))
        Next I
        
        ' append the Password
        For I = 0 To 6
            buf(send_len) = psw(I)
            send_len = send_len + 1
        Next I
        
        ' send the block
        If (TMBlockStream(SHandle, buf(0), send_len) <> send_len) Then
            ReadPageCRC77 = False
            Exit Function
        End If
    Else
        ' previous read was the last page
        crc16 = 0
    End If
    
    ' send last byte of password and enable strong pullup
    Dummy = TMOneWireLevel(SHandle, 0, 1, 2)
    Dummy = TMTouchByte(SHandle, psw(7))
    
    ' delay for read
    Sleep (3)
    
    ' turn off strong pullup
    Dummy = TMOneWireLevel(SHandle, 0, 0, 0)
  
    ' read the data bytes and one CRC byte
    For I = 0 To 64
        buf(I) = &HFF
    Next I
    
    ' send the block
    If (TMBlockStream(SHandle, buf(0), 65) <> 65) Then
        ReadPageCRC77 = False
        Exit Function
    End If
    
    ' calculate CRC on bytes read (except last byte)
    For I = 0 To 63
        Dummy = DoCrc16(buf(I))
    Next I
    
    ' check first CRC byte
    If (Dummy And &HFF) <> ((Not buf(64)) And &HFF) Then
        ReadPageCRC77 = False
        Exit Function
    End If
    
    ' copy data to buffer
    For I = 0 To 63
        data(I) = buf(I)
    Next I
      
    ReadPageCRC77 = True
End Function

' ------------------------------------------------------------------------------
' WriteScratchpad77 - Write the scratchpad of the DS1977. If full scratchpad
'      written then also check CRC16
'
'   SHandle - session handle
'   ROMID() - Array of Integers representing DS1977 ROM ID to write
'   str_add - address to write
'   wrdata() - array of bytes, data to write
'   wr_len  - number of bytes to write
'
' Returns TRUE if write to scratchpad was successful
'
Function WriteScratchpad77(SHandle As Long, ByRef ROMID() As Integer, str_add As Integer, ByRef wrdata() As Byte, wr_len As Integer) As Boolean
    Dim buf(80) As Byte
    Dim Dummy As Long
    Dim send_len As Integer
    Dim I As Integer
    Dim check_crc As Boolean
    
    ' set device ROM ID
    Dummy = TMRom(SHandle, state_buffer(0), ROMID(0))
    
    ' select the device
    If (TMAccess(SHandle, state_buffer(0)) <> 1) Then
        WriteScratchpad77 = False
        Exit Function
    End If
        
    ' block for write scratchpad
    buf(0) = &HF   ' write scratchpad command
    buf(1) = str_add And &HFF  ' TA1
    buf(2) = Int(str_add / 256) And &HFF ' TA2
    send_len = 3
    
    ' append the data
    For I = 0 To wr_len - 1
        buf(send_len) = wrdata(I)
        send_len = send_len + 1
    Next I
    
    ' check if full page (can utilize CRC)
    If (((str_add + wr_len) Mod 64) = 0) Then
        buf(send_len) = &HFF
        buf(send_len + 1) = &HFF
        send_len = send_len + 2
        check_crc = True
    Else
        check_crc = False
    End If
    
    ' send the block
    If (TMBlockStream(SHandle, buf(0), send_len) <> send_len) Then
        WriteScratchpad77 = False
        Exit Function
    End If
    
    ' verify echo of bytes was correct
    For I = 0 To wr_len - 1
        If (wrdata(I) <> buf(I + 3)) Then
            WriteScratchpad77 = False
            Exit Function
        End If
    Next I
    
    ' check if crc is available
    If (check_crc) Then
        crc16 = 0
        For I = 0 To send_len - 1
            Dummy = DoCrc16(buf(I))
        Next I
        
        If Dummy <> 45057 Then ' B001 hex
            WriteScratchpad77 = False
            Exit Function
        End If
    End If
    
    WriteScratchpad77 = True

End Function

' ------------------------------------------------------------------------------
' ReadScratchPad77 - read the scratchpad of the DS1977 and verify CRC16
'
'   SHandle - session handle
'   ROMID() - Array of Integers representing DS1977 ROM ID to read
'   addr - ByRef Integer address, will be set by this function
'   data() - byte buffer to store data read from scratchpad
'   ES - ByRef Integer to store ES byte
'   rd_len - ByRef Integer to store length of data in scratchpad
'
' Return TRUE if scratchpad read successfully and CRC16 verified
'
Function ReadScratchPad77(SHandle As Long, ByRef ROMID() As Integer, ByRef addr As Integer, ByRef data() As Byte, ByRef ES As Integer, ByRef rd_len As Integer) As Boolean
    Dim buf(80) As Byte
    Dim Dummy As Long
    Dim send_len As Integer
    Dim I As Integer
    Dim check_crc As Boolean
    Dim num_crc As Integer
    
    ' set device ROM ID
    Dummy = TMRom(SHandle, state_buffer(0), ROMID(0))
    
    ' select the device
    If (TMAccess(SHandle, state_buffer(0)) <> 1) Then
        ReadScratchPad77 = False
        Exit Function
    End If
        
    ' block for read scratchpad
    buf(0) = &HAA   ' read scratchpad command
    
    ' append the bytes to read, TA1,TA2,ES,64 data,2 crc
    For I = 1 To 69
        buf(I) = &HFF
    Next I
    send_len = 70
    
    ' send the block
    If (TMBlockStream(SHandle, buf(0), send_len) <> send_len) Then
        ReadScratchPad77 = False
        Exit Function
    End If
    
    ' get the starting offset to see when the crc will show up
    addr = buf(1)
    addr = (addr Or ((buf(2) * 256) And &HFF00)) And &HFFFF
    num_crc = 68 - (addr And &H3F)
    
    ' check crc
    crc16 = 0
    For I = 0 To num_crc + 1
        Dummy = DoCrc16(buf(I))
    Next I
   
    If Dummy <> 45057 Then ' B001 hex
        ReadScratchPad77 = False
        Exit Function
    End If
 
    ' extract the page data
    ES = buf(3)
    rd_len = ES - (addr And &H3F) + 1
    For I = 0 To rd_len - 1
        data(I) = buf(I + 4)
    Next I
    
    ReadScratchPad77 = True
End Function

' ------------------------------------------------------------------------------
' CopyScratchPad77 - copy the scratchpad of the DS1977 and verify complete
'
'   SHandle - session handle
'   ROMID() - Array of Integers representing DS1977 ROM ID to copy
'   psw() - Password array
'   addr - address to copy to (used in writeScratchpad)
'   ES - ES byte representing the ending offset (from readScratchpad)
'   cp_len - length of data to copy
'
' Return TRUE if scratchpad copy successful and toggle detected
'
Function CopyScratchPad77(SHandle As Long, ByRef ROMID() As Integer, ByRef psw() As Byte, addr As Integer, ES As Integer, cp_len As Integer) As Boolean
    Dim buf(80) As Byte
    Dim Dummy As Long
    Dim send_len As Integer
    Dim I As Integer
    
    ' select the device
    If (TMAccess(SHandle, state_buffer(0)) <> 1) Then
        CopyScratchPad77 = False
        Exit Function
    End If
        
    ' block to start copy sequence
    buf(0) = &H99  ' read memory with password command
    buf(1) = addr And &HFF  ' TA1
    buf(2) = Int(addr / 256) And &HFF ' TA2
    buf(3) = ES ' ES
    send_len = 4

    ' append the Password (except last byte)
    For I = 0 To 6
        buf(send_len) = psw(I)
        send_len = send_len + 1
    Next I
    
    ' send the block
    If (TMBlockStream(SHandle, buf(0), send_len) <> send_len) Then
        CopyScratchPad77 = False
        Exit Function
    End If
    
    ' send last byte of password and enable strong pullup
    Dummy = TMOneWireLevel(SHandle, 0, 1, 2)
    Dummy = TMTouchByte(SHandle, psw(7))
    
    ' delay for copy
    Sleep (24)
    
    ' turn off strong pullup
    Dummy = TMOneWireLevel(SHandle, 0, 0, 0)
  
    If (TMTouchByte(SHandle, &HFF) <> &HAA) Then
        CopyScratchPad77 = False
        Exit Function
    End If
  
    CopyScratchPad77 = True
    
End Function
' ------------------------------------------------------------------------------
' WritePage77 - write a page of data to the DS1977
'
'   SHandle - session handle
'   ROMID() - Array of Integers representing DS1977 ROM ID to copy
'   psw() - Password array
'   page - page number to write (0 to 510)
'   data() - data array to write
'   data_len - length of data in bytes (max 64)
'
' Returns TRUE if page was successfully written
'
Function WritePage77(SHandle As Long, ByRef ROMID() As Integer, ByRef psw() As Byte, page As Integer, ByRef data() As Byte, data_len As Integer) As Boolean
    Dim rd_len As Integer
    Dim ES As Integer
    
    ' write scratchpad
    If (WriteScratchpad77(SHandle, ROM, page * 64, data, data_len) <> True) Then
        WritePage77 = False
        Exit Function
    End If
    
    ' if it was not a full page then check contents with a readscratchpad
    If (data_len <> 64) Then
        If (ReadScratchPad77(SHandle, ROM, page * 64, data, ES, rd_len) <> True) Then
            WritePage77 = False
            Exit Function
        End If
    Else
        ES = data_len - 1
        rd_len = data_len
    End If
    
    ' copy the scratchpad to the destination
    If (CopyScratchPad77(SHandle, ROM, psw, page * 64, ES, rd_len) <> True) Then
        WritePage77 = False
        Exit Function
    End If
    
    WritePage77 = True
    
End Function

'

Function FindFirstFamily(fam As Integer, ByRef ROM() As Integer) As Boolean
    Dim I As Integer, flag As Integer
    
    '// Set up to find first device
    If TMFamilySearchSetup(SHandle, state_buffer(0), fam) = 1 Then
        flag = TMNext(SHandle, state_buffer(0))
        If flag = 1 Then
            '// Read the ROM number
            ROM(0) = 0
            flag = TMRom(SHandle, state_buffer(0), ROM(0))
            '// Check for the correct type
            If ((fam And &H7F) = (ROM(0) And &H7F)) Then
                FindFirstFamily = True
            Else
                FindFirstFamily = False
            End If
        Else
            FindFirstFamily = False
        End If
    Else
        FindFirstFamily = False
    End If
End Function

'// Routine that is executed first
Sub Main()
    Dim buf As String * 150
    Dim NBuf As String, Dummy As Long, temp As String
    Dim RtSize As Long, RtType As Long, GlobKey As Long
    Dim cstring(150) As Byte
    Dim I As Integer, setupdone As Integer, state As Integer, level As Integer
    Dim romstr As String

    For I = 0 To 510
        Swvb32.PageCombo.AddItem Str(I)
    Next
    
    '// get the default port
    If (TMReadDefaultPort(PortNum, PortType) <> 1) Then
        MsgBox ("Error: Failed to read default 1-Wire Net")
    End If

    '// Display the port number and type
    temp = "Port Type: " + CStr(PortType) + "     Port Number: "
    temp = temp + CStr(PortNum)
    Swvb32.Label10.Caption = temp
    '// get the main and hardware driver version
    I = Get_Version(buf)
    NBuf = Left$(buf, InStr(1, buf, Chr$(0), 1) - 1)
    Swvb32.Label2.Caption = NBuf
    If (TMGetTypeVersion(PortType, buf) > 0) Then
        NBuf = Left$(buf, InStr(1, buf, Chr$(0), 1) - 1)
        Swvb32.Label3.Caption = Swvb32.Label3.Caption + Chr$(&HD) + NBuf
    End If

    '// Start session
    SHandle = -1
    While (SHandle <= 0)
        SHandle = TMExtendedStartSession(PortNum, PortType, vbNullString)
    Wend

    '// Setup must be successful
    If (TMSetup(SHandle) = 1) Then
        If FindFirstFamily(family, ROM) <> True Then
            MsgBox ("DS1977 Note Found!")
        End If
    Else
        MsgBox ("Error: Failed to setup 1-Wire Net! Trying again.")
        setupdone = TMSetup(SHandle)
        If FindFirstFamily(family, ROM) <> True Then
            MsgBox ("DS1977 Note Found!")
        End If
    End If

    '// Create a string out of the ROM number
    romstr = ""
    For I = 7 To 0 Step -1
        If (ROM(I) <= &HF) Then romstr = romstr + "0"
        romstr = romstr + Hex$(ROM(I))
    Next I
    Swvb32.RomLabel.Caption = romstr

    '// Release the 1-wire
    TMEndSession (SHandle)

    Swvb32.Show
End Sub
